<?php

# Personal Weblog 0.9.1
#
# Copyright 2001  Mark Pulford <mark@kyne.com.au>
# This file is subject to the terms and conditions of the GNU General Public
# License. Read the file COPYING found in this archive for details, or
# visit http://www.gnu.org/copyleft/gpl.html
#
# See http://www.kyne.com.au/~mark/software/weblog.php for the lastest
# version.

class Weblog {
	var $admin;
	var $dbh, $dbvar, $dbname;
	var $broken;
	var $valid_cgi;

	# Acceptable cgi variables (when prefix with "wl_")
	var $mode, $eid, $tid, $password, $name, $icon, $value, $title, $body;
	var $more, $update, $offset, $search, $topic, $delopt, $before, $start;

	function insert()
	{
		if($this->broken) {
			$this->print_error("Cannot insert, not initialised");
			return;
		}
	
		if(isset($this->password) && !$this->admin) {
			echo "<p><b>Error: incorrect password<p>\n";
		}

		echo $this->getvar("header")."\n";

		if(!isset($this->mode)) {
			$this->entry_search($this->topic, $this->offset, $this->search, $this->before, $this->start);
		} else switch($this->mode) {
		case "search":
			$this->entry_search($this->topic, $this->offset, $this->search, $this->before, $this->start);
			break;
		case "set variables":
			$this->var_update($this->value);
			break;
		case "more":
			$this->entry_display($this->eid);
			break;
		case "edit entry":
			$this->entry_edit($this->eid);
			break;
		case "save entry":
			$this->entry_update($this->eid, $this->title, $this->tid, $this->body, $this->more, $this->update);
			break;
		case "delete entry":
			$this->entry_delete($this->eid);
			break;
		case "add entry":
			$this->entry_add($this->title, $this->tid, $this->body, $this->more);
			break;
		case "show add entry":
			$this->entry_edit_new();
			break;
		case "delete topic":
			$this->topic_delete($this->tid, $this->delopt);
			break;
		case "change topic":
			$this->topic_update($this->tid, $this->name, $this->icon);
			break;
		case "add topic":
			$this->topic_add($this->name, $this->icon);
			break;
		case "show edit topics":
			$this->topic_edit();
			break;
		case "show edit setup":
			$this->var_edit();
			break;
		case "login":
			$this->login();
			break;
		case "logout":
			$this->logout();
			$this->entry_search($this->topic, $this->offset, $this->search, $this->before, $this->start);
			break;
		default:
			$this->print_error("Unknown mode: $mode");
		}

		echo $this->getvar("footer")."\n";
	}

	function admin()
	{
		global $PHP_SELF;

		if(!$this->admin) {
			$this->print_error("not logged in");
			echo "<p><form action=\"$PHP_SELF\" method=\"POST\">\n";
			$this->print_form_vars(array_diff($this->valid_cgi,array("password")));
			echo "<input type=\"password\" name=\"wl_password\">\n";
			echo "<input type=\"submit\" value=\"Login\">\n";
			echo "</form>\n";
			$this->print_back();
		}

		return $this->admin;
	}

	function search_string()
	{
		$vars = array();
		foreach (array("offset","topic","before", "start") as $v)
			if($this->$v)
				array_push($vars, "wl_$v=".$this->$v);
		if($this->search) {
			$s = ereg_replace(" ", "+", $this->search);
			array_push($vars, "wl_search=$s");
		}
		return join('&', $vars);
	}

	function print_back()
	{
		global $PHP_SELF;

		$q = $this->search_string();
		if($q)
			$q = "?$q";

		echo "<p><a href=\"$PHP_SELF$q\">back</a>\n";
	}

	function topic_update($tid, $name, $icon)
	{
		if(!$this->admin())
			return;

		if(!$tid) {
			$this->print_error("no topic selected");
			$this->print_back();
			return;
		}

		if($name)
			$terms = "name='$name'";
		if($icon) {
			if($terms)
				$terms .= ", ";
			$terms .= "icon='$icon'";
		}
		if(!$terms) {
			$this->print_error("At least one of icon or name must be set");
			$this->print_back();
			return;
		}

		if(!$this->dbquery("UPDATE topics SET $terms WHERE tid=$tid"))
			return;
		if(mysql_affected_rows())
			echo "<h1>Topic changed</h1>\n";
		else
			$this->print_error("topic add failed");

		$this->print_back();
	}

	function topic_add($name, $icon)
	{
		if(!$this->admin())
			return;

		if(!$name or !$icon) {
			$this->print_error("Both name and icon must be set");
			$this->print_back();
			return;
		}

		if(!$this->dbquery("INSERT INTO topics VALUES (NULL, '$name', '$icon')"))
			return;
		if(mysql_affected_rows())
			echo "<h1>Topic added</h1>\n";
		else
			$this->print_error("topic add failed");

		$this->print_back();
	}

	function topic_delete($tid, $delopt)
	{
		echo "<h1>Deleting topic</h1>\n";

		if(!$this->admin())
			return;

		if(!$tid) {
			$this->print_error("no topic selected");
			$this->print_back();
			return;
		}

		# Verify "delopt" exists unless it is zero
		# 0 => use "tid" for a dummy lookup.
		$v = $delopt;
		if(!$v)
			$v = $tid;
		$tname = $this->topic_verify($v);
		if(!$tname)
			return;

		if($delopt) {
			$q = "UPDATE entries SET tid=$delopt WHERE tid=$tid";
			echo "<p>Moving entries to $tname\n";
		} else {
			$q = "DELETE FROM entries WHERE tid=$tid";
			echo "<p>Deleting any existing entries for $tname\n";
		}

		if(!$this->dbquery($q)) {
			if(!$this->dbquery("UNLOCK TABLES")) {
				$this->print_error("CRITICAL! Failed to unlock tables");
				return;
			}
		}
		if(!$this->dbquery_unlock("DELETE FROM topics WHERE tid=$tid", $aff))
			return;

		if($aff)
			echo "<p><b>Success</b>\n";
		else
			$this->print_error("unable to delete topic");
		$this->print_back();
	}

	function print_form_vars($cgi_names)
	{
		foreach($cgi_names as $v) {
			if(isset($this->$v)) {
				if(is_array($this->$v)) {
					foreach ($this->$v as $k2 => $v2)
						echo "<input type=\"hidden\" name=\"wl_{$v}[{$k2}]\" value=\"".htmlspecialchars($v2)."\">\n";
				} else {
					echo "<input type=\"hidden\" name=\"wl_$v\" value=\"".htmlspecialchars($this->$v)."\">\n";
				}
			}
		}
	}

	function print_search_details()
	{
		$this->print_form_vars(array("offset", "topic", "search", "before", "start"));
	}

	function topic_edit()
	{
		global $PHP_SELF;

		if(!$this->admin())
			return;

		echo "<h1>Edit topics</h1>\n";

		$r = $this->dbquery("SELECT * FROM topics");
		if(!$r)
			return;
		echo "<p><form action=\"$PHP_SELF\" method=\"POST\">\n";
		$this->print_search_details();
		echo "<table border=1 cellpadding=3>\n";
		echo "<tr><th>name</th><th>icon</th><th>sel</th></tr>\n";
		while($a = mysql_fetch_array($r)) {
			echo "<tr>";
			echo "<td>".$a["name"]."</td>\n";
			echo "<td><img src=\"{$a["icon"]}\"><a href=\"".$a["icon"]."\">".$a["icon"]."</a></td>\n";
			echo "<td><input type=\"radio\" name=\"wl_tid\" value=\"{$a["tid"]}\"></td>";
			echo "</tr>\n";
		}
		echo "</table>\n";
		echo "<input type=\"radio\" name=\"wl_tid\" value=\"\" checked> select none<br>\n";

		echo "Name: <input type=\"text\" name=\"wl_name\" maxlength=\"80\"> (add/change option)<br>\n";
		echo "Icon: <input type=\"text\" name=\"wl_icon\" maxlength=\"128\"> (add/change option)<br>\n";
		echo "<input type=\"submit\" name=\"wl_mode\" value=\"change topic\">\n";
		echo "<input type=\"submit\" name=\"wl_mode\" value=\"add topic\"><br>\n";
		echo "Move or delete entries currently in topic: ";
		$this->print_topic_select("delopt", -1, "Delete");
		echo "<input type=\"submit\" name=\"wl_mode\" value=\"delete topic\">\n";
		echo "</form>\n";
		$this->print_back();
	}

	function var_update($value)
	{
		echo "<h2>Updated setup</h2>\n";

		if(!$this->admin())
			return;

		foreach($value as $k => $v)
			$this->setvar($k, $v);

		$this->print_back();
	}

	# image_link("next", "gfx/next.gif", array("mode"=>"next", "a"=>"b"));
	# $name must not be a potential CGI variable
	function image_link($name, $gfx, $vars)
	{
		global $PHP_SELF;

		echo "<form action=\"$PHP_SELF\" method=\"POST\">";
		foreach($vars as $k => $v)
			echo "<input type=\"hidden\" name=\"wl_$k\" value=\"$v\">";
		echo "<input type=\"image\" src=\"$gfx\" name=\"$name\" ".$this->getvar("nav_icon_attrs").">";
		echo "</form>";
	}

	function var_edit()
	{
		global $PHP_SELF;

		if(!$this->admin())
			return;

		echo "<h1>Edit weblog setup</h1>\n";

		$r = $this->dbquery("select * from vars where editable!='no' order by name asc");
		if(!$r)
			return;

		echo "<p><form action=\"$PHP_SELF\" method=\"POST\">\n";
		$this->print_search_details();
		while($a = mysql_fetch_array($r)) {
			echo "<h2>{$a["name"]}</h2>\n";
			if($a["help"])
				echo "<p>{$a["help"]}\n";
			echo "<p>";
			if($a["editable"] == "single") {
				echo "<input type=\"text\" name=\"wl_value[{$a["name"]}]\" value=\"".htmlspecialchars($a["value"])."\" size=\"60\">\n";
			} else {
				echo "<textarea name=\"wl_value[{$a["name"]}]\" rows=\"5\" cols=\"60\">".htmlspecialchars($a["value"])."</textarea>\n";
			}
		}
		echo "<p><input type=\"submit\" name=\"wl_mode\" value=\"set variables\">\n";
		echo "</form>\n";
		$this->print_back();
	}

	function entry_edit_new()
	{
		global $PHP_SELF;

		if(!$this->admin())
			return;

		echo "<h1>Add entry form</h1>\n";
		echo "<form action=\"$PHP_SELF\" method=\"POST\">\n";
		$this->print_search_details();
		echo "Title: <input type=\"text\" name=\"wl_title\" maxlength=\"80\" size=\"60\"><br>\n";
		echo "Topic:&nbsp;";
		$this->print_topic_select("tid", -1, "");
		echo "<br>\n";
		echo "Body:<br>\n";
		echo "<textarea name=\"wl_body\" rows=\"5\" cols=\"60\"></textarea><br>\n";
		echo "More:<br>\n";
		echo "<textarea name=\"wl_more\" rows=\"5\" cols=\"60\"></textarea><br>\n";
		echo "<input type=\"submit\" name=\"wl_mode\" value=\"add entry\"><br>\n";
		echo "</form>\n";
		$this->print_back();
	}

	function entry_update($eid, $title, $tid, $body, $more, $update)
	{
		if(!$this->admin())
			return;

		if(!$tid or !$eid) {
			$this->print_error("topic or entry id not set");
			return;
		}

		if(!$this->topic_verify($tid))
			return;

		$q = "UPDATE entries SET title='$title', tid=$tid, body='$body', more='$more'";
		if($update == "yes")
			$q .= ", updated=".time();
		$q .= " WHERE eid=$eid";

		if(!$this->dbquery_unlock($q, $aff))
			return;

		if(!$aff) {
			echo "<p><b>Error</b><br>\n";
			echo "No entry found to update.\n";
		} else {
			echo "<p><b>Entry successfully updated</b>\n";
		}
		$this->print_back();
	}

	function entry_delete($eid)
	{
		if(!$this->admin())
			return;

		if(!$this->dbquery("DELETE FROM entries WHERE eid=$eid"))
			return;
		if(mysql_affected_rows() == 1) {
			echo "<p><b>Entry deleted</b>\n";
		} else {
			echo "<p><b>Error</b><br>\n";
			echo "No entry found to delete.\n";
		}
		$this->print_back();
	}

	function entry_edit($eid)
	{
		global $PHP_SELF;

		if(!$this->admin())
			return;

		echo "<h1>Edit entry</h1>\n";

		$r = $this->dbquery("SELECT * FROM entries WHERE eid=$eid");
		if(!$r)
			return;
		if(!mysql_num_rows($r)) {
			echo "<p><b>Error</b>: entry not found\n";
			return;
		}

		$entry = mysql_fetch_array($r);

		echo "<form action=\"$PHP_SELF\" method=\"POST\">\n";
		$this->print_search_details();
		echo "<input type=\"hidden\" name=\"wl_eid\" value=\"$eid\">\n";
		echo "Title: <input type=\"text\" name=\"wl_title\" value=\"".htmlspecialchars($entry["title"])."\" maxlength=\"80\" size=\"60\"><br>\n";
		echo "Topic:&nbsp;";
		$this->print_topic_select("tid", $entry["tid"], "");
		echo "<br>\n";
		echo "Body:<br>\n";
		echo "<textarea name=\"wl_body\" rows=\"5\" cols=\"60\">".htmlspecialchars($entry["body"])."</textarea><br>\n";
		echo "More:<br>\n";
		echo "<textarea name=\"wl_more\" rows=\"5\" cols=\"60\">".htmlspecialchars($entry["more"])."</textarea><br>\n";
		echo "<input type=\"checkbox\" name=\"wl_update\" value=\"yes\"> Update timestamp<br>\n";
		echo "<input type=\"submit\" name=\"wl_mode\" value=\"save entry\"><br>\n";
		echo "</form>\n";
		$this->print_back();
	}

	function entry_search($topic, $offset, $search, $before, $start)
	{
		$sep = $this->getvar("separator");
		$max = $this->getvar("max_articles");

		$q = "SELECT * from entries,topics WHERE entries.tid=topics.tid ";

		if($start) {
			$r = $this->dbquery("SELECT updated from entries WHERE eid=$start");
			if(!$r)
				return;
			if(!mysql_num_rows($r)) {
				echo "<b>No entries found.</b>\n";
				echo "$sep\n";
				$this->print_control($topic, $search, $offset, false);
				return;
			}
			$arr = mysql_fetch_array($r);
			$q .= "AND updated<=".$arr["updated"]." ";
		}
		if($before)
			$q .= "AND updated<=$before ";
		if(0<$topic)
			$q .= "AND topics.tid=$topic ";
		if($search) {
			$keywords = split(" +", $search);
			foreach($keywords as $k)
				$q .= "AND entries.body LIKE '%$k%' ";
		}
		$q .= "ORDER BY updated DESC ";
		if(0<$max) {
			if(!$offset)
				$offset = 0;

			# Search for an extra entry. Don't display the extra
			# entry, it is used to determine whether a next icon
			# is needed.
			$q .= "LIMIT $offset, ".($max+1)." ";
		}
		$r = $this->dbquery($q);
		if(!$r)
			return;
		$rows = mysql_num_rows($r);

		if($rows > $max) {
			$show_next = 1;
			$rows = $max;
		}

		if($rows == 0) {
			echo "<b>No entries found.</b>\n";
			echo "$sep\n";
		} else {
			$this->print_entries($r, $rows);
		}

		$this->print_control($topic, $search, $offset, $show_next);
	}

	function entry_display($eid)
	{
		$r = $this->dbquery("SELECT * FROM entries,topics WHERE eid=$eid and entries.tid=topics.tid");
		if(!$r)
			return;
		if(!mysql_num_rows($r)) {
			echo "<b>No entry found.</b>\n";
		} else {
			$mf = $this->getvar("more");
			echo $this->subst_tokens($mf, mysql_fetch_array($r))."\n";
			$this->print_back();
			if($this->admin)
				$this->entry_print_admin($eid);
		}
		echo $this->getvar("separator")."\n";
		$this->print_control(0, "", 0, 0);
	}

	function entry_print_admin($eid)
	{
		global $PHP_SELF;

		$q = $this->search_string();
		if($q)
			$q = "&$q";
		echo "<p><b>Admin</b>: ";
		echo "<a href=\"$PHP_SELF?wl_mode=edit+entry&wl_eid=$eid$q\">[edit]</a>\n";
		echo "<a href=\"$PHP_SELF?wl_mode=delete+entry&wl_eid=$eid$q\">[delete]</a>\n";
	}

	function sanitize_input()
	{
		# name, icon, title, body, more, value
		# - Used in db searches, but they are slash escaped
		# mode, password, update
		# - not used in db searches

		# Sanitize. remove non alphanumeric. shrink spaces.
		# remove prefix/suffix spaces.
		if(isset($this->search)) {
			$this->search = eregi_replace("[^a-z0-9 ]", "", $this->search);
			$this->search = eregi_replace(" +", " ", $this->search);
			$this->search = eregi_replace("(^ | $)", "", $this->search);
			# Only keep first 3 keywords
			# Do this here so the search box contains the updated
			# search string.
			if(ereg("([^ ]+ [^ ]+ [^ ]+) .*", $this->search, $reg)) {
				$this->search = $reg[1];
			}
		}
		# Ensure numeric CGI vars are valid.
		foreach (array("topic","delopt","tid","eid","offset","before","start") as $v)
			if(isset($this->$v)) {
				$this->$v = eregi_replace("[^0-9]", "", $this->$v);
				# Ensure a blanked var will return 0 as a string
				if(!$this->$v)
					$this->$v = 0;
			}
	}

	function print_entries($r, $rows)
	{
		$af = $this->getvar("article");
		$sep = $this->getvar("separator");

		for($i=0; $i<$rows; $i++) {
			$arr = mysql_fetch_array($r);
			echo $this->subst_tokens($af, $arr)."\n";
			if($this->admin)
				$this->entry_print_admin($arr["eid"]);
			echo "$sep\n";
		}
	}

	function subst_tokens($format, $entry)
	{
		global $PHP_SELF;

		# Create morelink for @MORE@
		if($entry["more"]) {
			$ml = $this->getvar("morelink");
			$q = $this->search_string();
			if($q)
				$q = "&$q";
			$ml = ereg_replace("@URL@", "$PHP_SELF?wl_mode=more&wl_eid={$entry["eid"]}$q", $ml);
		} else {
			$ml = "";
		}

		if($entry["created"] != $entry["updated"]) {
			$upd = $this->getvar("update_text");
			$upd = ereg_replace("@DATE@", date($this->getvar("updated_date"), $entry["updated"]), $upd);
		} else {
			$upd = "";
		}

		$f = ereg_replace("@TITLE@", $entry["title"], $format);
		$f = ereg_replace("@CREATED@", date($this->getvar("created_date"), $entry["created"]), $f);
		$f = ereg_replace("@UPDATED@", $upd, $f);
		$f = ereg_replace("@ICON@", "<a href=\"$PHP_SELF?wl_topic={$entry["tid"]}\"><img src=\"{$entry["icon"]}\" alt=\"Topic: {$entry["name"]}\" align=\"left\" ".$this->getvar("topic_icon_attrs")."></a>", $f);
		$f = ereg_replace("@BODY@", $entry["body"], $f);
		$f = ereg_replace("@ID@", $entry["eid"], $f);
		$f = ereg_replace("@MORELINK@", $ml, $f);
		$f = ereg_replace("@MORE@", $entry["more"], $f);

		return $f;
	}

	function print_offset_button($name, $var, $off)
	{
		if($off < 0)
			$off = 0;
		$this->image_link("[$name]", $this->getvar($var), array("search"=>$this->search, "topic"=>$this->topic, "offset"=>$off, "before"=>$this->before, "start"=>$this->start));
	}

	function print_blank_icon()
	{
		echo "<img src=\"";
		echo $this->getvar("blank_icon");
		echo "\" alt=\"\" ".$this->getvar("nav_icon_attrs").">";
	}

	# Show search box, topic select, ...
	function print_control($topic, $search, $offset, $show_next)
	{
		global $PHP_SELF;

		$max = $this->getvar("max_articles");

		echo "<center><table><tr><td>";
		if($offset > 0)
			$this->print_offset_button("previous", "prev_icon", $offset - $max);
		else
			$this->print_blank_icon();
		echo "</td><td>";
		echo "<form method=\"POST\" action=\"$PHP_SELF\">\n";
		echo "&nbsp;Topic:&nbsp;";
		$this->print_topic_select("topic", $topic, "All");
		echo "&nbsp;";
		echo "<input type=\"text\" name=\"wl_search\" value=\"$search\">&nbsp;";
		echo "<input type=\"submit\" name=\"wl_mode\" value=\"search\">&nbsp;";
		echo "</form>";
		echo "</td><td>";
		if($show_next)
			$this->print_offset_button("next", "next_icon", $offset + $max);
		else
			$this->print_blank_icon();
		echo "</td></tr></table>";
		echo "</center>\n";

		if($this->admin) {
			$q = $this->search_string();
			if($q)
				$q = "&$q";
			echo "<p><b>Admin</b>:\n";
			echo "<a href=\"$PHP_SELF?wl_mode=show+add+entry$q\">[add entry]</a>\n";
			echo "<a href=\"$PHP_SELF?wl_mode=show+edit+topics$q\">[edit topics]</a>\n";
			echo "<a href=\"$PHP_SELF?wl_mode=show+edit+setup$q\">[edit setup]</a>\n";
			echo "<a href=\"$PHP_SELF?wl_mode=logout$q\">[log out]</a>\n";
		}
	}

	function dbquery($sql)
	{
		# Must ensure correct DB is selected. If some other PHP
		# code has run it might have changed the db in between
		# Weblog() and insert().
		if(!mysql_select_db($this->dbname)) {
			$this->print_error("cannot select correct database");
			return false;
		}

		$result = @mysql_query($sql, $this->dbh);
		if($result) {
			return $result;
		} else {
			$this->print_error("query failed: ".mysql_error());
			$this->print_back();
			return false;
		}
	}

	function print_error($err)
	{
		$h = headers_sent();
	
		if($err)
			$err = ": $err";
		if(!$h) {
			echo "<html><head><title>Critical error</title></head>\n";
			echo "<body>";
		}
		echo "<p><b>ERROR</b>$err\n";
		if(!$h) {
			echo "</body></html>\n";
			exit;
		}
	}

	# Verify the topic exists.
	# NOTE: It leaves the table locked!
	# Locking both entries and topics for write is overkill, but this
	# code doesn't get used much.
	function topic_verify($tid)
	{
		if(!$this->dbquery("LOCK TABLES entries WRITE, topics WRITE"))
			return "";

		$r = $this->dbquery("SELECT tid,name FROM topics WHERE tid=$tid");
		if(!$r || !mysql_num_rows($r)) {
			$this->print_error("Topic not found.");
			$this->print_back();
			if(!$this->dbquery("UNLOCK TABLES"))
				$this->print_error("CRITICAL. Cannot unlock table!");
			return "";
		}

		$a = mysql_fetch_array($r);
		return $a["name"];
	}

	function dbquery_unlock($sql, &$affected)
	{
		$r = $this->dbquery($sql);

		# Ugly, but if we don't do this we lose the information.
		$affected = mysql_affected_rows();
		
		if(!$this->dbquery("UNLOCK TABLES")) {
			$this->print_error("CRITICAL! Cannot unlock tables");
			return false;
		}

		return $r;
	}

	function entry_add($title, $tid, $body, $more)
	{
		global $PHP_SELF;

		if(!$this->admin())
			return;

		if(!$tid) {
			$this->print_error("query incomplete");
			return;
		}

		if(!$this->topic_verify($tid))
			return;

		$tm = time();
		$q = "insert into entries values (NULL, '$title', $tid, $tm, $tm, '$body','$more')";
		if(!$this->dbquery_unlock($q, $aff))
			return;
		
		echo "<h1>Entry added</h1>\n";

		$this->print_back();
	}

	# Variables are assumed to exist and are required to run.
	function getvar($name)
	{
		if(isset($this->dbvar) && isset($this->dbvar[$name]))
			return $this->dbvar[$name];

		$this->print_error("fatal, cannot load variable $name");
		exit;
	}

	function setvar($name, $value)
	{
		if($this->getvar($name) == $value)
			return true;

		$this->dbvar[$name] = $value;

		$q = "UPDATE vars SET value='$value' WHERE name='$name'";
		if(!$this->dbquery($q))
			return false;
		if(mysql_affected_rows() != 1) {
			$this->print_error("Cannot set $name/$value");
			return false;
		}

		return true;
	}

	function print_topic_select($cgi, $t, $field)
	{
	        $r = $this->dbquery("SELECT tid,name from topics ORDER BY name");
		if(!$r)
			return;

	        echo "<select name=\"wl_$cgi\">\n";
		if($field)
		        echo "<option value=\"0\">$field\n";
	        while($a = mysql_fetch_array($r)) {
	                echo "<option value=\"{$a["tid"]}\"";
	                if($t == $a["tid"])
	                        echo " SELECTED";
	                echo ">".$a["name"]."\n";
	        }
	        echo "</select>";
	}

	function login()
	{
		global $PHP_SELF;

		echo "<h2>Admin log in</h2>\n";
		echo "<p><form action=\"$PHP_SELF\" method=\"POST\">\n";
		$this->print_form_vars(array_diff($this->valid_cgi,array("password", "mode")));
		echo "<input type=\"password\" name=\"wl_password\">\n";
		echo "<input type=\"submit\" value=\"Login\">\n";
		echo "</form>\n";
		$this->print_back();
	}

	function logout()
	{
		$this->setvar("login_expire", -1);
		$this->admin = false;
	}

	function current_login()
	{
		global $HTTP_COOKIE_VARS;

		$t = $this->getvar("login_expire");
		if($t < time())
			return false;

		if($HTTP_COOKIE_VARS["sess"] != $this->getvar("login_cookie"))
			return false;

		return true;
	}

	# called before the headers have been sent
	function load_db_vars()
	{
		$r = $this->dbquery("SELECT name,value FROM vars");
		if(!$r || !mysql_num_rows($r))
			$this->print_error("cannot load variable");

		while($a = mysql_fetch_array($r))
			$this->dbvar[$a["name"]] = $a["value"];
	}

	function load_cgi_vars()
	{
		global $HTTP_GET_VARS, $HTTP_POST_VARS;

		$arr = array_merge($HTTP_GET_VARS, $HTTP_POST_VARS);

		foreach($arr as $k => $v)
			if(ereg("^wl_(.+)", $k, $r) && in_array($r[1], $this->valid_cgi))
				$this->$r[1] = $v;
	}

	function enter_session($sess)
	{
		$exp = time() + $this->getvar("login_timeout");
		setcookie("sess", $sess, $exp);
		$this->setvar("login_cookie", $sess);
		$this->setvar("login_expire", $exp);
		$this->admin = true;
	}

	# The Weblog class must be instantiated at the top of the page
	# before any data is sent. This way it can modify the HTTP headers.
	#
	# Checks for a valid new or current login and updates the session cookie
	# and expiry.
	# Sets the class variable $admin to true on a successful login.
	function Weblog($dbname, $dbhost, $dbuser, $dbpass)
	{
		$this->valid_cgi = array(
			"mode","topic","search","offset","before","start",
			"eid","tid","name","icon","value",
			"title","body","more",
			"password","update","delopt");

		if(headers_sent()) {
			print_error("Weblog construct must be called before the headers have been sent");
			$this->broken = 1;
			return;
		}

		$this->dbh = @mysql_connect($dbhost,$dbuser,$dbpass);
		if(!$this->dbh)
			$this->print_error("Unable to connect to MySQL server. Check Weblog() arguments.");

		$this->dbname = $dbname;

		$this->load_cgi_vars();
		$this->sanitize_input();
		$this->load_db_vars();

		# Check for login status
		if($this->current_login())
			$this->enter_session($this->getvar("login_cookie"));
		else if($this->password == $this->getvar("password"))
			$this->enter_session($this->gen_session_key());
		else
			$this->admin = false;
	}

	function gen_session_key()
	{
		mt_srand( ((double)microtime() + posix_getpid()) * 1000000 );
		return mt_rand().mt_rand().mt_rand();
	}
}

?>
